@@ -345,7 +345,7 @@ $$ LANGUAGE 'sql' IMMUTABLE STRICT;
 CREATE or REPLACE FUNCTION set_dav_property( TEXT, INTEGER, TEXT, TEXT ) RETURNS BOOLEAN AS $$
 DECLARE
   path ALIAS FOR $1;
-  user ALIAS FOR $2;
+  change_user ALIAS FOR $2;
   key ALIAS FOR $3;
   value ALIAS FOR $4;
   tmp_int INT;
@@ -360,9 +360,9 @@ BEGIN
   END IF;
   SELECT changed_by INTO tmp_int FROM property WHERE dav_name = path AND property_name = key;
   IF FOUND THEN
-    UPDATE property SET changed_by=user, changed_on=current_timestamp, property_value=value WHERE dav_name = path AND property_name = key;
+    UPDATE property SET changed_by=change_user, changed_on=current_timestamp, property_value=value WHERE dav_name = path AND property_name = key;
   ELSE
-    INSERT INTO property ( dav_name, changed_by, changed_on, property_name, property_value ) VALUES( path, user, current_timestamp, key, value );
+    INSERT INTO property ( dav_name, changed_by, changed_on, property_name, property_value ) VALUES( path, change_user, current_timestamp, key, value );
   END IF;
   RETURN TRUE;
 END;
@@ -774,18 +774,18 @@ DECLARE
   in_priv TEXT;
   out_bits BIT(24);
   i INT;
-  allprivs BIT(24);
+  all_privs BIT(24);
   start INT;
   finish INT;
 BEGIN
   out_bits := 0::BIT(24);
-  allprivs := ~ out_bits;
+  all_privs := ~ out_bits;
   SELECT array_lower(raw_privs,1) INTO start;
   SELECT array_upper(raw_privs,1) INTO finish;
   FOR i IN start .. finish  LOOP
     SELECT out_bits | privilege_to_bits(raw_privs[i]) INTO out_bits;
-    IF out_bits = allprivs THEN
-      RETURN allprivs;
+    IF out_bits = 65535::BIT(24) THEN
+      RETURN all_privs;
     END IF;
   END LOOP;
   RETURN out_bits;
@@ -1206,16 +1206,21 @@ DECLARE
   in_old_sync_token ALIAS FOR $1;
   in_collection_id ALIAS FOR $2;
   tmp_int INT8;
+  old_modification_time sync_tokens.modification_time%TYPE;
 BEGIN
   IF in_old_sync_token > 0 THEN
-    SELECT 1 INTO tmp_int FROM sync_changes
-            WHERE collection_id = in_collection_id
-              AND sync_time > (SELECT modification_time FROM sync_tokens WHERE sync_token = in_old_sync_token)
-            LIMIT 1;
+    SELECT modification_time INTO old_modification_time FROM sync_tokens WHERE sync_token = in_old_sync_token;
     IF NOT FOUND THEN
       -- They are in an inconsistent state: we return NULL so they can re-start the process
       RETURN NULL;
     END IF;
+    SELECT 1 INTO tmp_int FROM sync_changes WHERE collection_id = in_collection_id AND sync_time > old_modification_time LIMIT 1;
+    IF NOT FOUND THEN
+      -- Ensure we return the latest sync_token we have for this collection, since there are
+      -- no changes.
+	  SELECT sync_token INTO tmp_int FROM sync_tokens WHERE collection_id = in_collection_id ORDER BY modification_time DESC LIMIT 1;
+      RETURN tmp_int;
+    END IF;
   END IF;
   SELECT nextval('sync_tokens_sync_token_seq') INTO tmp_int;
   INSERT INTO sync_tokens(collection_id, sync_token) VALUES( in_collection_id, tmp_int );
