@@ -45,18 +45,18 @@ DECLARE
   in_priv TEXT;
   out_bits BIT(24);
   i INT;
-  all BIT(24);
+  allprivs BIT(24);
   start INT;
   finish INT;
 BEGIN
   out_bits := 0::BIT(24);
-  all := ~ out_bits;
+  allprivs := ~ out_bits;
   SELECT array_lower(raw_privs,1) INTO start;
   SELECT array_upper(raw_privs,1) INTO finish;
   FOR i IN start .. finish  LOOP
     SELECT out_bits | privilege_to_bits(raw_privs[i]) INTO out_bits;
-    IF out_bits = all THEN
-      RETURN all;
+    IF out_bits = allprivs THEN
+      RETURN allprivs;
     END IF;
   END LOOP;
   RETURN out_bits;
@@ -334,13 +334,22 @@ CREATE TABLE addressbook_resource (
   n TEXT, -- Name Surname;First names
   note TEXT,
   org TEXT,
-  url TEXT
+  url TEXT,
+  fburl TEXT,
+  caladruri TEXT,
+  caluri TEXT
 );
 
 CREATE TABLE addressbook_address_adr (
   dav_id INT8 NOT NULL REFERENCES caldav_data(dav_id) ON UPDATE CASCADE ON DELETE CASCADE,
   type TEXT,
-  adr TEXT,
+  box_no TEXT,
+  unit_no TEXT,
+  street_address TEXT,
+  locality TEXT,
+  region TEXT,
+  postcode TEXT,
+  country TEXT,
   property TEXT -- The full text of the property
 );
 
@@ -366,7 +375,8 @@ CREATE TABLE calendar_alarm (
   summary TEXT,
   description TEXT,
   next_trigger TIMESTAMP WITH TIME ZONE,
-  component TEXT -- The full text of the component
+  component TEXT, -- The full text of the component
+  trigger_state CHAR DEFAULT 'N' -- 'N' => 'New/Needs setting', 'A' = 'Active', 'O' = 'Old'
 );
 
 CREATE TABLE calendar_attendee (
@@ -378,7 +388,9 @@ CREATE TABLE calendar_attendee (
   role TEXT,
   rsvp BOOLEAN,
   property TEXT, -- The full text of the property
+  attendee_state TEXT, -- Internal DAViCal processing state
+  weak_etag TEXT, -- The week_etag applying for this attendee state
   PRIMARY KEY ( dav_id, attendee )
 );
 
-SELECT new_db_revision(1,2,8, 'Août' );
+SELECT new_db_revision(1,2,9, 'Septembre' );
