@@ -49,6 +49,8 @@ $c->base_url = preg_replace('#/[^/]+\.php.*$#', '', $_SERVER['SCRIPT_NAME']);
 $c->base_directory = preg_replace('#/[^/]*$#', '', $_SERVER['DOCUMENT_ROOT']);
 $c->default_privileges = array('read-free-busy', 'schedule-deliver');
 
+$c->enable_auto_schedule = true;
+
 $c->stylesheets = array( $c->base_url.'/davical.css' );
 $c->images      = $c->base_url . '/images';
 
@@ -166,7 +168,8 @@ init_gettext( 'davical', $c->locale_path );
 *
 */
 $c->code_version = 0;
-$c->version_string = '0.9.9.3'; // The actual version # is replaced into that during the build /release process
+$c->want_awl_version = 0.46;
+$c->version_string = '0.9.9.4'; // The actual version # is replaced into that during the build /release process
 if ( isset($c->version_string) && preg_match( '/(\d+)\.(\d+)\.(\d+)(.*)/', $c->version_string, $matches) ) {
   $c->code_major = $matches[1];
   $c->code_minor = $matches[2];
@@ -200,9 +203,10 @@ if ( $qry->Exec('always',__LINE__,__FILE__) && $row = $qry->Fetch() ) {
 }
 
 
-$_known_users_name = array();
-$_known_users_id   = array();
-$_known_users_pid  = array();
+$_known_users_name  = array();
+$_known_users_id    = array();
+$_known_users_pid   = array();
+$_known_users_email = array();
 
 function _davical_get_principal_query_cached( $where, $parameter ) {
   global $c, $session, $_known_users_name, $_known_users_id, $_known_users_pid;
@@ -225,6 +229,7 @@ function _davical_get_principal_query_cached( $where, $parameter ) {
       $_known_users_name[$row->username]      = $row;
       $_known_users_id[$row->user_no]         = $row;
       $_known_users_pid[$row->principal_id]   = $row;
+      $_known_users_email[$row->email]        = $row;
     }
     return $row;
   }
@@ -253,12 +258,7 @@ function getUserByName( $username, $use_cache = true ) {
 function getUserByEMail( $email, $use_cache = true ) {
   global $_known_users_name;
 
-  if ( $use_cache ) {
-    /** We don't actually maintain a cache-access by e-mail, since it's rare */
-    foreach( $_known_users_name AS $k => $v ) {
-      if ( strtolower($email) == strtolower($v->email) ) return $v;
-    }
-  }
+  if ( $use_cache && isset( $_known_users_email[$email] ) ) return $_known_users_email[$email];
   return _davical_get_principal_query_cached( 'lower(email) = lower(:param)', $email );
 }
 
@@ -358,6 +358,7 @@ function ConstructURL( $partial_path, $force_script = false ) {
   $partial_path = str_replace( '%2F', '/', $partial_path);
 
   if ( ! isset($c->_url_script_path) ) {
+    $c->protocol_server_port_script = str_replace( 'index.php', 'caldav.php', $c->protocol_server_port_script);
     $c->_url_script_path = (preg_match('#/$#', $c->protocol_server_port_script) ? 'caldav.php' : '');
     $c->_url_script_path = $c->protocol_server_port_script . $c->_url_script_path;
   }
@@ -412,7 +413,7 @@ function DeconstructURL( $url, $force_script = false ) {
 */
 function ISODateToHTTPDate( $isodate ) {
   // Use strtotime since strptime is not available on Windows platform.
-  return( gmstrftime('%a, %d %b %Y %T GMT', strtotime($isodate)) );
+  return( gmstrftime('%a, %d %b %Y %H:%M:%S GMT', strtotime($isodate)) );
 }
 
 /**
@@ -496,7 +497,7 @@ function bits_to_privilege( $raw_bits, $resourcetype = 'resource' ) {
   if ( ($raw_bits &  16) != 0 ) $out_priv[] = 'DAV::read-acl';
   if ( ($raw_bits &  32) != 0 ) $out_priv[] = 'DAV::read-current-user-privilege-set';
   if ( ($raw_bits & 256) != 0 ) $out_priv[] = 'DAV::write-acl';
-  if ( ($resourcetype == 'calendar' || $resourcetype == 'proxy') && ($raw_bits & 512) != 0 ) $out_priv[] = 'urn:ietf:params:xml:ns:caldav:read-free-busy';
+  if ( ($resourcetype == 'calendar' || $resourcetype == 'proxy' || $resourcetype == '*') && ($raw_bits & 512) != 0 ) $out_priv[] = 'urn:ietf:params:xml:ns:caldav:read-free-busy';
 
   if ( ($raw_bits & 198) != 0 ) {
     if ( ($raw_bits & 198) == 198 ) $out_priv[] = 'DAV::write';
@@ -506,14 +507,14 @@ function bits_to_privilege( $raw_bits, $resourcetype = 'resource' ) {
     if ( ($raw_bits & 128) != 0 ) $out_priv[] = 'DAV::unbind';
   }
 
-  if ( $resourcetype == 'schedule-inbox' && ($raw_bits & 7168) != 0 ) {
+  if ( ($resourcetype == 'schedule-inbox' || $resourcetype == '*') && ($raw_bits & 7168) != 0 ) {
     if ( ($raw_bits & 7168) == 7168 ) $out_priv[] = 'urn:ietf:params:xml:ns:caldav:schedule-deliver';
     if ( ($raw_bits & 1024) != 0 ) $out_priv[] = 'urn:ietf:params:xml:ns:caldav:schedule-deliver-invite';
     if ( ($raw_bits & 2048) != 0 ) $out_priv[] = 'urn:ietf:params:xml:ns:caldav:schedule-deliver-reply';
     if ( ($raw_bits & 4096) != 0 ) $out_priv[] = 'urn:ietf:params:xml:ns:caldav:schedule-query-freebusy';
   }
 
-  if ( $resourcetype == 'schedule-outbox' && ($raw_bits & 57344) != 0 ) {
+  if ( ($resourcetype == 'schedule-outbox' || $resourcetype == '*') && ($raw_bits & 57344) != 0 ) {
     if ( ($raw_bits & 57344) == 57344 ) $out_priv[] = 'urn:ietf:params:xml:ns:caldav:schedule-send';
     if ( ($raw_bits &  8192) != 0 ) $out_priv[] = 'urn:ietf:params:xml:ns:caldav:schedule-send-invite';
     if ( ($raw_bits & 16384) != 0 ) $out_priv[] = 'urn:ietf:params:xml:ns:caldav:schedule-send-reply';
