@@ -1,4 +1,71 @@
 <?php
+/** @TODO: work out something more than true/false returns for dependency checks */
+
+/**
+ * We put many of these checks before we even try to load always.php so that we
+ * can try and do some diagnostic work to ensure it will load OK.
+ */
+function check_pgsql() {
+  return function_exists('pg_connect');
+}
+
+function check_pdo() {
+  return class_exists('PDO');
+}
+
+function check_pdo_pgsql() {
+  global $loaded_extensions;
+
+  if ( !check_pdo() ) return false;
+  return isset($loaded_extensions['pdo_pgsql']);
+}
+
+function check_gettext() {
+  global $phpinfo, $loaded_extensions;
+
+  if ( !function_exists('gettext') ) return false;
+  return isset($loaded_extensions['gettext']);
+}
+
+function check_suhosin_server_strip() {
+  global $loaded_extensions;
+
+  if ( !isset($loaded_extensions['suhosin']) ) return true;
+  if ( ini_get('suhosin.server.strip') == "0" ) {
+    return true;
+  }
+
+  return false;
+}
+
+function check_magic_quotes_gpc() {
+  global $loaded_extensions;
+
+  if ( ini_get('magic_quotes_gpc') == "0" ) {
+    return true;
+  }
+
+  return false;
+}
+
+function check_magic_quotes_runtime() {
+  global $loaded_extensions;
+
+  if ( ini_get('magic_quotes_runtime') == "0" ) {
+    return true;
+  }
+
+  return false;
+}
+
+function do_error( $errormessage ) {
+  printf("<p class='error'>%s</p>", $errormessage );  
+}
+$loaded_extensions = array_flip(get_loaded_extensions());
+if ( !check_pgsql() )     do_error("PHP 'pgsql' functions are not available" );
+if ( !check_pdo() )       do_error("PHP 'PDO' module is not available" );
+if ( !check_pdo_pgsql() ) do_error("The PDO drivers for PostgreSQL are not available" );
+
 
 include("./always.php");
 include("DAViCalSession.php");
@@ -11,45 +78,11 @@ ob_end_clean( );
 $phpinfo = preg_replace( '{^.*?<body>}s', '', $phpinfo);
 $phpinfo = preg_replace( '{</body>.*?$}s', '', $phpinfo);
 
-$loaded_extensions = array_flip(get_loaded_extensions());
-
-/** @TODO: work out something more than true/false returns for dependency checks */
-function check_pgsql() {
-  return function_exists('pg_connect');
-}
 
 if ( check_pgsql() ) {
   $session->LoginRequired( (isset($c->restrict_setup_to_admin) && $c->restrict_setup_to_admin ? 'Admin' : null ) );
 }
 
-function check_pdo() {
-  return class_exists('PDO');
-}
-
-function check_pdo_pgsql() {
-  global $phpinfo, $loaded_extensions;
-
-  if ( !class_exists('PDO') ) return false;
-  return isset($loaded_extensions['pdo_pgsql']);
-}
-
-function check_gettext() {
-  global $phpinfo, $loaded_extensions;
-
-  if ( !function_exists('gettext') ) return false;
-  return isset($loaded_extensions['gettext']);
-}
-
-function check_suhosin_server_strip() {
-  global $phpinfo, $loaded_extensions;
-
-  if ( !isset($loaded_extensions['suhosin']) ) return true;
-  if ( ini_get('suhosin.server.strip') == "0" ) {
-    return true;
-  }
-
-  return false;
-}
 
 include("interactive-page.php");
 include("page-header.php");
@@ -77,6 +110,16 @@ function check_davical_version() {
 }
 
 
+function check_awl_version() {
+  global $c;
+  
+  if ( function_exists('awl_version') ) {
+    return ( $c->want_awl_version == awl_version() ? true : sprintf( "want %f", $c->awl_library_version) );
+  }
+  return false;
+}
+
+
 function build_site_statistics() {
   $principals  = translate('No. of Principals');
   $collections = translate('No. of Collections');
@@ -108,11 +151,14 @@ EOTABLE;
 $dependencies = array(
   translate('Current DAViCal version '). $c->version_string => 'check_davical_version',
   translate('DAViCal DB Schema version '). implode('.',$c->want_dbversion) => 'check_schema_version',
+  translate('AWL Library version '). $c->want_awl_version => 'check_awl_version',
   translate('PHP PDO module available') => 'check_pdo',
   translate('PDO PostgreSQL drivers') => 'check_pdo_pgsql',
   translate('PHP PostgreSQL available') => 'check_pgsql',
   translate('GNU gettext support') => 'check_gettext',
-  translate('Suhosin "server.strip" disabled') => 'check_suhosin_server_strip' /*,
+  translate('Suhosin "server.strip" disabled') => 'check_suhosin_server_strip',
+  translate('PHP Magic Quotes GPC off') => 'check_magic_quotes_gpc',
+  translate('PHP Magic Quotes runtime off') => 'check_magic_quotes_runtime' /*,
   'YAML' => 'php5-syck' */
 );
 
@@ -130,7 +176,7 @@ foreach( $dependencies AS $k => $v ) {
 $want_dbversion = implode('.',$c->want_dbversion);
 
 $heading_setup = translate('Setup');
-$paragraph_setup = translate('Currently this page does very little.  Suggestions or patches to make it do more useful stuff will be gratefully received.');
+$paragraph_setup = translate('This page primarily checks the environment needed for DAViCal to work correctly.  Suggestions or patches to make it do more useful stuff will be gratefully received.');
 
 $heading_versions = translate('Current Versions');
 if ( check_schema_version() != true )
