@@ -27,7 +27,7 @@ class CalDAVPrincipal
   /**
   * @var The home URL of the principal
   */
-  var $url;
+  private $url;
 
   /**
   * @var Identifies whether this principal exists in the DB yet
@@ -249,7 +249,7 @@ class CalDAVPrincipal
     $this->write_proxy_for = null;
     $this->read_proxy_for = null;
 
-    dbg_error_log( 'principal', ' User: %s (%d) URL: %s, Home: %s, By Email: %d', $this->username, $this->user_no, $this->url, $this->by_email );
+    dbg_error_log( 'principal', ' User: %s (%d) URL: %s, Home: %s, By Email: %d', $this->username, $this->user_no, $this->url, $this->principal_url, $this->by_email );
   }
 
 
@@ -435,6 +435,38 @@ class CalDAVPrincipal
   }
 
 
+  /**
+  * Return an arbitrary property
+  * @return string The name of the arbitrary property
+  */
+  function GetProperty( $property_id ) {
+
+    switch( $property_id ) {
+      case 'DAV::resource-id':
+        if ( $this->exists && $this->principal_id > 0 )
+          ConstructURL('/.resources/'.$this->principal_id);
+        else
+          return null;
+        break;
+    }
+
+    if ( isset($this->{$property_id}) ) {
+      if ( ! is_object($this->{$property_id}) ) return $this->{$property_id};
+      return clone($this->{$property_id});
+    } 
+    return null;
+  }
+
+
+  /**
+  * Return the URL for this principal
+  * @return string The principal-URL, or null if they don't exist
+  */
+  function url() {
+    return ($this->exists ? $this->url : null );
+  }
+
+
   /**
   * Get the calendar_home_set, as lazily as possible
   */
@@ -504,8 +536,12 @@ class CalDAVPrincipal
   * Return the privileges bits for the current session user to this resource
   */
   function Privileges() {
+    global $session;
     if ( !isset($this->privileges) ) $this->privileges = 0;
     if ( is_string($this->privileges) ) $this->privileges = bindec( $this->privileges );
+    if ( $this->_is_group && in_array(ConstructURL('/'.$session->username.'/'), $this->GroupMemberSet()) ) {
+      $this->privileges |=  privilege_to_bits( array('DAV::read', 'DAV::read-current-user-privilege-set') ); 
+    }
     return $this->privileges;
   }
 
@@ -624,10 +660,6 @@ class CalDAVPrincipal
         $reply->DAVElement( $prop, 'owner', $reply->href( $this->principal_url ) );
         break;
 
-      case 'DAV::principal-collection-set':
-        $reply->DAVElement( $prop, 'principal-collection-set', $reply->href( ConstructURL('/') ) );
-        break;
-
       // Empty tag responses.
       case 'DAV::alternate-URI-set':
         $prop->NewElement( $reply->Tag($tag));
