@@ -485,20 +485,20 @@ EOSQL;
   function FetchPrincipal() {
     if ( isset($this->principal) ) return;
     $this->principal = new CalDAVPrincipal( array( "path" => $this->bound_from() ) );
-    if ( $this->_is_principal && $this->principal->Exists() ) {
-      $this->exists = true;
-      $this->unique_tag = '"'.$this->principal->dav_etag.'"';
-      $this->created = $this->principal->created;
-      $this->modified = $this->principal->modified;
-      $this->resourcetypes = '<DAV::collection/><DAV::principal/>';
-      $this->resource_id = $this->principal->principal_id;
-      $this->collection = $this->principal->AsCollection();
-      $this->user_no = $this->principal->user_no;
-    }
-    elseif ( $this->_is_principal ) {
-      $this->exists = false;
+    if ( $this->_is_principal ) {
+      $this->exists = $this->principal->Exists();
       $this->collection->dav_name = $this->dav_name;
       $this->collection->type = 'principal';
+      if ( $this->exists ) {
+        $this->displayname = $this->principal->GetProperty('displayname');
+        $this->unique_tag = '"'.$this->principal->dav_etag.'"';
+        $this->created = $this->principal->created;
+        $this->modified = $this->principal->modified;
+        $this->resourcetypes = '<DAV::collection/><DAV::principal/>';
+        $this->resource_id = $this->principal->principal_id;
+        $this->collection = $this->principal->AsCollection();
+        $this->user_no = $this->principal->user_no;
+      }
     }
   }
 
@@ -588,7 +588,7 @@ EOQRY;
     if ( $this->IsPrincipal() ) {
       if ( !isset($this->principal) ) $this->FetchPrincipal();
       $this->privileges = $this->principal->Privileges();
-      dbg_error_log( 'DAVResource', 'Privileges of "%s" for user accessing "%s"', $this->privileges, $this->principal->username() );
+      dbg_error_log( 'DAVResource', 'Privileges of "%s" for user accessing principal "%s"', $this->privileges, $this->principal->username() );
       return;
     }
 
@@ -656,15 +656,19 @@ EOQRY;
 
 
   /**
-  * Check if we have the needed privilege or send an error response.
+  * Check if we have the needed privilege or send an error response.  If the user does not have the privileges then
+  * the call will not return, and an XML error document will be output.
   *
   * @param string $privilege The name of the needed privilege.
+  * @param boolean $any Whether we accept any of the privileges. The default is true, unless the requested privilege is 'all', when it is false.
   */
   function NeedPrivilege( $privilege, $any = null ) {
     global $request;
 
+    // Do the test
     if ( $this->HavePrivilegeTo($privilege, $any) ) return;
 
+    // They failed, so output the error
     $request->NeedPrivilege( $privilege, $this->dav_name );
     exit(0);  // Unecessary, but might clarify things
   }
@@ -728,17 +732,21 @@ EOQRY;
           $this->supported_methods['HEAD'] = '';
           $this->supported_methods['MKTICKET'] = '';
           $this->supported_methods['DELTICKET'] = '';
+          $this->supported_methods['ACL'] = '';
           break;
         case 'collection':
           $this->supported_methods['MKTICKET'] = '';
           $this->supported_methods['DELTICKET'] = '';
+          $this->supported_methods['BIND'] = '';
+          $this->supported_methods['ACL'] = '';
         case 'principal':
           $this->supported_methods['GET'] = '';
-          $this->supported_methods['PUT'] = '';
           $this->supported_methods['HEAD'] = '';
           $this->supported_methods['MKCOL'] = '';
           $this->supported_methods['MKCALENDAR'] = '';
           $this->supported_methods['PROPPATCH'] = '';
+          $this->supported_methods['BIND'] = '';
+          $this->supported_methods['ACL'] = '';
           break;
       }
     }
@@ -1014,6 +1022,17 @@ EOQRY;
   }
 
 
+  /**
+  * Returns the URL of our resource
+  */
+  function url() {
+    if ( !isset($this->dav_name) ) {
+      throw Exception("What! How can dav_name not be set?");
+    }
+    return ConstructURL($this->dav_name);
+  }
+
+
   /**
   * Returns the dav_name of the resource in our internal namespace
   */
@@ -1065,10 +1084,7 @@ EOQRY;
   */
   function principal_url() {
     if ( !isset($this->principal) ) $this->FetchPrincipal();
-    if ( $this->principal->Exists() ) {
-      return $this->principal->principal_url;
-    }
-    return null;
+    return $this->principal->url();
   }
 
 
@@ -1232,8 +1248,21 @@ EOQRY;
           return $type_list;
         }
 
+      case 'resource':
+        if ( !isset($this->resource) ) $this->FetchResource();
+        return clone($this->resource);
+        break;
+
+      case 'principal':
+        if ( !isset($this->principal) ) $this->FetchPrincipal();
+        return clone($this->principal);
+        break;
+
       default:
-        if ( isset($this->{$name}) ) return $this->{$name};
+        if ( isset($this->{$name}) ) {
+          if ( ! is_object($this->{$name}) ) return $this->{$name};
+          return clone($this->{$name});
+        } 
         if ( $this->_is_principal ) {
           if ( !isset($this->principal) ) $this->FetchPrincipal();
           if ( isset($this->principal->{$name}) ) return $this->principal->{$name};
@@ -1250,7 +1279,11 @@ EOQRY;
           if ( isset($this->principal->{$name}) ) return $this->principal->{$name};
           if ( isset($this->collection->{$name}) ) return $this->collection->{$name};
         }
-//        dbg_error_log( 'DAVResource', ':GetProperty: Failed to find property "%s" on "%s".', $name, $this->dav_name );
+        if ( isset($this->{$name}) ) {
+          if ( ! is_object($this->{$name}) ) return $this->{$name};
+          return clone($this->{$name});
+        } 
+        // dbg_error_log( 'DAVResource', ':GetProperty: Failed to find property "%s" on "%s".', $name, $this->dav_name );
     }
 
     return $value;
@@ -1473,6 +1506,10 @@ EOQRY;
         $prop->NewElement('supported-privilege-set', $request->BuildSupportedPrivileges($reply) );
         break;
 
+      case 'DAV::principal-collection-set':
+        $prop->NewElement( 'principal-collection-set', $reply->href( ConstructURL('/') ) );
+        break;
+
       case 'DAV::current-user-principal':
         $prop->NewElement('current-user-principal', $reply->href( $request->principal->principal_url ) );
         break;
