@@ -40,6 +40,18 @@ function CreateHomeCalendar( $username ) {
   $parent_path = "/".$username."/";
   $calendar_path = $parent_path . $c->home_calendar_name."/";
   $dav_etag = md5($usr->user_no . $calendar_path);
+  $qry = new AwlQuery( 'SELECT 1 FROM collection WHERE dav_name = :dav_name', array( ':dav_name' => $calendar_path) );
+  if ( $qry->Exec() ) {
+    if ( $qry->rows() > 0 ) {
+      $c->messages[] = i18n("Home calendar already exists.");
+      return true;
+    }
+  }
+  else {
+    $c->messages[] = i18n("There was an error writing to the database.");
+    return false;
+  }
+
   $sql = 'INSERT INTO collection (user_no, parent_container, dav_name, dav_etag, dav_displayname, is_calendar, created, modified, resourcetypes) ';
   $sql .= 'VALUES( :user_no, :parent_container, :calendar_path, :dav_etag, :displayname, true, current_timestamp, current_timestamp, :resourcetypes );';
   $params = array(
@@ -112,7 +124,7 @@ function UpdateUserFromExternal( &$usr ) {
   }
   else
     $type = "INSERT";
-
+    
   $params = array();
   if ( $type != 'INSERT' ) $params[':user_no'] = $usr->user_no;
   $qry = new AwlQuery( sql_from_object( $usr, $type, 'usr', 'WHERE user_no= :user_no' ), $params );
@@ -129,6 +141,12 @@ function UpdateUserFromExternal( &$usr ) {
     $qry->Exec('Login',__LINE__,__FILE__);
     CreateHomeCalendar($usr->username);
   }
+  else if ( $usr->fullname != $old->{'fullname'} ) {
+    // Also update the displayname if the fullname has been updated.
+    $qry->QDo( 'UPDATE principal SET displayname=:new_display WHERE user_no=:user_no',
+                    array(':new_display' => $usr->fullname, ':user_no' => $usr->user_no)
+             );
+  }
 }
 
 
