@@ -273,27 +273,29 @@ $bound_from = $target_collection->bound_from();
 if ( !$target_collection->Exists() ) {
   $request->DoResponse( 404 );
 }
-if ( ! ($target_collection->IsCalendar() || $target_collection->IsSchedulingCollection()) ) {
-  $request->DoResponse( 403, translate('The calendar-query report must be run against a calendar or a scheduling collection') );
-}
-
-/**
-* @todo Once we are past DB version 1.2.1 we can change this query more radically.  The best performance to
-* date seems to be:
-*   SELECT caldav_data.*,calendar_item.* FROM collection JOIN calendar_item USING (collection_id,user_no)
-*         JOIN caldav_data USING (dav_id) WHERE collection.dav_name = '/user1/home/'
-*              AND caldav_data.caldav_type = 'VEVENT' ORDER BY caldav_data.user_no, caldav_data.dav_name;
-*/
 
 $params = array();
 $where = ' WHERE caldav_data.collection_id = ' . $target_collection->resource_id();
+
+if ( ! ($target_collection->IsCalendar() || $target_collection->IsSchedulingCollection()) ) {
+  if ( !(isset($c->allow_recursive_report) && $c->allow_recursive_report) || $target_collection->IsSchedulingCollection() ) {
+    $request->DoResponse( 403, translate('The calendar-query report must be run against a calendar or a scheduling collection') );
+  }
+  /**
+   * We're here because they allow recursive reports, and this appears to be such a location.
+   */
+  $where = 'WHERE (collection.dav_name ~ :path_match ';
+  $where .= 'OR collection.collection_id IN (SELECT bound_source_id FROM dav_binding WHERE dav_binding.dav_name ~ :path_match)) ';
+  $params = array( ':path_match' => '^'.$target_collection->bound_from() );
+}
+
 if ( is_array($qry_filters) ) {
   dbg_log_array( "calquery", "qry_filters", $qry_filters, true );
   $components = array();
   $filter_fragment =  SqlFilterFragment( $qry_filters, $components );
   if ( $filter_fragment !== false ) {
     $where .= ' '.$filter_fragment['sql'];
-    $params = $filter_fragment['params'];
+    $params = array_merge( $params, $filter_fragment['params']);
   }
 }
 if ( $target_collection->Privileges() != privilege_to_bits('DAV::all') ) {
@@ -308,8 +310,8 @@ if ( isset($c->hide_older_than) && intval($c->hide_older_than > 0) ) {
   $where .= " AND calendar_item.dtstart > (now() - interval '".intval($c->hide_older_than)." days') ";
 }
 
-$sql = 'SELECT * FROM caldav_data INNER JOIN calendar_item USING(dav_id,user_no,dav_name)'. $where;
-if ( isset($c->strict_result_ordering) && $c->strict_result_ordering ) $sql .= " ORDER BY dav_id";
+$sql = 'SELECT caldav_data.*,calendar_item.*  FROM collection INNER JOIN caldav_data USING(collection_id) INNER JOIN calendar_item USING(dav_id) '. $where;
+if ( isset($c->strict_result_ordering) && $c->strict_result_ordering ) $sql .= " ORDER BY caldav_data.dav_id";
 $qry = new AwlQuery( $sql, $params );
 if ( $qry->Exec("calquery",__LINE__,__FILE__) && $qry->rows() > 0 ) {
   while( $calendar_object = $qry->Fetch() ) {
@@ -320,6 +322,7 @@ if ( $qry->Exec("calquery",__LINE__,__FILE__) && $qry->rows() > 0 ) {
       if ( $need_expansion ) {
         $vResource = new vComponent($calendar_object->caldav_data);
         $expanded = expand_event_instances($vResource, $expand_range_start, $expand_range_end);
+        if ( $expanded->ComponentCount() == 0 ) continue;
         $calendar_object->caldav_data = $expanded->Render();
       }
       $responses[] = calendar_to_xml( $properties, $calendar_object );
