@@ -68,34 +68,43 @@ foreach( $mg_hrefs AS $k => $v ) {
    * anything up to the matching request->path (which will include any http...) and then
    * put the $bound_from prefix back on.
    */
-  $href = $bound_from . preg_replace( "{^.*\E$request->path\Q}", '', rawurldecode($v->GetContent()) );
-  dbg_error_log("REPORT", "Reporting on href '%s'", $href );
+  $rawurl = rawurldecode($v->GetContent());
+  $path_pos = strpos($rawurl,$request->path);
+  if ( $path_pos === false ) {
+    $href = $bound_from . $rawurl;
+  }
+  else {
+    $href = $bound_from . substr( $rawurl, $path_pos + strlen($request->path));
+  }
+  @dbg_error_log("REPORT", 'Reporting on href "%s"', $href );
   $href_in .= ($href_in == '' ? '' : ', ');
   $href_in .= ':href'.$k;
   $params[':href'.$k] = $href;
 }
 
 $where = " WHERE caldav_data.collection_id = " . $collection->resource_id();
-$where .= "AND caldav_data.dav_name IN ( $href_in ) ";
+$where .= " AND caldav_data.dav_name IN ( $href_in ) ";
 
 if ( $mode == 'caldav' ) {
   if ( $collection->Privileges() != privilege_to_bits('DAV::all') ) {
-    $where .= "AND (calendar_item.class != 'PRIVATE' OR calendar_item.class IS NULL) ";
+    $where .= " AND (calendar_item.class != 'PRIVATE' OR calendar_item.class IS NULL) ";
   }
 
   if ( isset($c->hide_TODO) && $c->hide_TODO && ! $collection->Privileges() == privilege_to_bits('all') ) {
-    $where .= "AND caldav_data.caldav_type NOT IN ('VTODO') ";
+    $where .= " AND caldav_data.caldav_type NOT IN ('VTODO') ";
   }
-  $sql = 'SELECT caldav_data.*,calendar_item.* FROM caldav_data
-                  LEFT JOIN calendar_item USING(dav_id, user_no, dav_name, collection_id)
-                  LEFT JOIN collection USING(collection_id)';
 }
-else if ( $mode == 'carddav' ) {
-  $sql = 'SELECT caldav_data.*, addressbook_resource.* FROM caldav_data
+$sql = 'SELECT calendar_item.*, addressbook_resource.*, caldav_data.* FROM caldav_data
+                  LEFT JOIN calendar_item USING(dav_id, user_no, dav_name, collection_id)
                   LEFT JOIN addressbook_resource USING(dav_id)
                   LEFT JOIN collection USING(collection_id)';
-}
 
+/**
+ * @todo: Add stanzas for missing rows, so we don't just return a blank multistatus but
+ * actually return <response> stanzas with a 404 for each absent href.  We could do
+ * this relatively easily with an array_flip($params) and remove each matching dav_name
+ * as we process it.
+ */
 if ( isset($c->strict_result_ordering) && $c->strict_result_ordering ) $where .= " ORDER BY caldav_data.dav_id";
 $qry = new AwlQuery( $sql . $where, $params );
 if ( $qry->Exec('REPORT',__LINE__,__FILE__) && $qry->rows() > 0 ) {
