@@ -121,6 +121,9 @@ function calendar_to_xml( $properties, $item ) {
         $contentlength = strlen($caldav_data);
         $prop->NewElement($k, $contentlength );
         break;
+      case 'getlastmodified':
+        $prop->NewElement($k, ISODateToHTTPDate($item->modified) );
+        break;
       case 'calendar-data':
         $reply->CalDAVElement($prop, $k, $caldav_data );
         break;
@@ -202,7 +205,7 @@ function component_to_xml( $properties, $item ) {
     case 'VCARD':
       $displayname = $item->fn;
       $type = 'vcard';
-      $contenttype = 'text/x-vcard';
+      $contenttype = 'text/vcard';
       break;
   }
   if ( isset($properties['calendar-data']) || isset($properties['displayname']) ) {
@@ -241,6 +244,9 @@ function component_to_xml( $properties, $item ) {
         $contentlength = strlen($caldav_data);
         $prop->NewElement($k, $contentlength );
         break;
+      case 'getlastmodified':
+        $prop->NewElement($k, ISODateToHTTPDate($item->modified) );
+        break;
       case 'calendar-data':
         if ( $type == 'calendar' ) $reply->CalDAVElement($prop, $k, $caldav_data );
         else $unsupported[] = $k;
