@@ -368,7 +368,7 @@ if ( $editor->Available() ) {
       $_POST['by_collection'] = $id;
       $to_principal = intval($_POST['to_principal']);
       $orig_to_id =  intval($_POST['orig_to_id']);
-      $grantrow->SetWhere( "by_collection=".qpg($id)." AND to_principal=$orig_to_id");
+      $grantrow->SetWhere( "by_collection=".$id." AND to_principal=$orig_to_id");
       if ( isset($_POST['grant_privileges']) ) {
         $privilege_bitpos = array_flip($privilege_names);
         $priv_names = array_keys($_POST['grant_privileges']);
@@ -514,7 +514,7 @@ EOTEMPLATE;
   $browser = new Browser(translate('Bindings to this Collection'));
   $browser->AddColumn( 'bind_id', translate('ID'), '', '' );
   $browser->AddHidden( 'b.dav_owner_id' );
-  $browser->AddColumn( 'bound_as', translate('Bound As'), '', '<td style="white-space:nowrap;">%s</td>', "'".$c->base_url.'/caldav.php'."' ||b.dav_name" );
+  $browser->AddColumn( 'bound_as', translate('Bound As'), '', '<td style="white-space:nowrap;">%s</td>', 'b.dav_name' );
   $browser->AddColumn( 'access_ticket_id', translate('Ticket ID'), '', '' );
   $browser->AddColumn( 'privs', translate('Privileges'), '', '', "privileges_list(privileges)" );
   $delurl = $c->base_url . '/admin.php?action=edit&t=principal&id=##dav_owner_id##&bind_id=##URL:bind_id##&subaction=delete_binding';
